/****************************************************************************
**  SCALASCA    http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2014                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2013                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include <config.h>
#include "CbData.h"

#include <cstddef>

#if defined(_MPI)
  #include <pearl/LocalData.h>
  #include <pearl/RemoteData.h>
#endif   // _MPI

using namespace std;
using namespace pearl;
using namespace scout;


//---------------------------------------------------------------------------
//
//  class CbData
//
//---------------------------------------------------------------------------

//--- Constructors & destructor ---------------------------------------------

/**
 *  @brief Constructor.
 *
 *  Creates a new callback data object.
 **/

CbData::CbData()
    : mLocal(NULL),
      mRemote(NULL),
      mInvLocal(NULL),
      mInvRemote(NULL),
      mDefs(NULL),
      mTrace(NULL),
      mSynchpointHandler(NULL),
      mTmapCacheHandler(NULL),
      mOmpEventHandler(NULL),
      mUpdateCallstack(true)
{
  #if defined(_MPI)
    mLocal     = new LocalData;
    mRemote    = new RemoteData;
    mInvLocal  = new LocalData;
    mInvRemote = new RemoteData;
  #endif   // _MPI
}


/**
 *  @brief Destructor.
 *
 *  Frees up all occupied resources and destroys the instance.
 **/
CbData::~CbData()
{
  #if defined(_MPI)
    delete mLocal;
    delete mRemote;
    delete mInvLocal;
    delete mInvRemote;
  #endif   // _MPI
}


//--- Pre- and postprocessing -----------------------------------------------

/**
 *  @brief Postprocessing after an event is handled.
 *
 *  Updates a number of attributes after the given @a event has been handled.
 *  In particular, the current callstack is updated and the local and remote
 *  event sets are cleared.
 *
 *  @param  event  Processed event
 **/
void
CbData::postprocess(const Event& event)
{
    // Update callstack
    if (mUpdateCallstack) {
        if (event->isOfType(GROUP_ENTER)) {
            mCallstack.push(event);
        } else if (event->isOfType(LEAVE)) {
            mCallstack.pop();
        }
    }

    // Clear event sets
  #if defined(_MPI)
    mLocal->clear();
    mRemote->clear();
    mInvLocal->clear();
    mInvRemote->clear();
  #endif   // _MPI
}
