#!/bin/sh

##
## This file is part of the Score-P software (http://www.score-p.org)
##
## Copyright (c) 2013,
## Technische Universitaet Dresden, Germany
##
## This software may be modified and distributed under the terms of
## a BSD-style license.  See the COPYING file in the package base
## directory for details.
##

set -e

cleanup()
{
    rm -rf test-trace-parallel-posix-*
}
trap cleanup EXIT
cleanup

getsubset()
{
    local n=$1
    local i=$2
    local ns=0
    local is=""
    for b in $(seq 0 $(($n - 1)))
    do
        if test $(((1 << $b) & $i)) -gt 0
        then
            ns=$((ns + 1))
            is="$is $b"
        fi
    done
    echo $is
}

tests=0
successes=0
fails=0
failed=
failed_sep=
nl='
'
for i in 1 2 3 4
do
    # write with local def files
    t=test-trace-parallel-posix-$i
    tests=$((tests + 1))
    writer_cmd="./test-writer -nt $i $t 1"
    test ${V-0} -eq 1 && echo "$i - Writing POSIX trace with $i rank(s) (w/ local defs): $writer_cmd"
    if $VALGRIND $writer_cmd
    then
        successes=$((successes + 1))
    else
        fails=$((fails + 1))
        failed="$failed$failed_sep$writer_cmd"
        failed_sep=$nl
        continue
    fi

    n=0
    for j in $(seq 1 $i) $((2 * i))
    do
        for k in $(seq 0 $(((1 << $i) - 1)))
        do
            s=$(getsubset $i $k)

            tests=$((tests + 1))
            n=$((n + 1))
            reader_cmd="./test-reader -nt $j $t.otf2 0 $s"
            test ${V-0} -eq 1 && echo "$i.$n - Reading with $j rank(s) |{$s}| out of $i rank(s) (w/o subgroups): $reader_cmd"
            if $VALGRIND $reader_cmd
            then
                successes=$((successes + 1))
            else
                fails=$((fails + 1))
                failed="$failed$writer_cmd$nl$reader_cmd"
                writer_cmd=
            fi

            tests=$((tests + 1))
            n=$((n + 1))
            reader_cmd="./test-reader -nt $j $t.otf2 1 $s"
            test ${V-0} -eq 1 && echo "$i.$n - Reading with $j rank(s) |{$s}| out of $i rank(s) (w/ subgroups): $reader_cmd"
            if $VALGRIND $reader_cmd
            then
                successes=$((successes + 1))
            else
                fails=$((fails + 1))
                failed="$failed$writer_cmd$nl$reader_cmd"
                writer_cmd=
            fi

            tests=$((tests + 1))
            n=$((n + 1))
            reader_cmd="./test-reader -nt $j $t.otf2 2 $s"
            test ${V-0} -eq 1 && echo "$i.$n - Reading with $j rank(s) |{$s}| out of $i rank(s) (w/ serial): $reader_cmd"
            if $VALGRIND $reader_cmd
            then
                successes=$((successes + 1))
            else
                fails=$((fails + 1))
                failed="$failed$writer_cmd$nl$reader_cmd"
                writer_cmd=
            fi
        done
    done

    # write without local def files
    t_nold="$t-nold"
    tests=$((tests + 1))
    writer_cmd="./test-writer -nt $i $t_nold 1"
    test ${V-0} -eq 1 && echo "$i - Writing POSIX trace with $i rank(s) (w/o local defs): $writer_cmd"
    if $VALGRIND $writer_cmd
    then
        successes=$((successes + 1))
    else
        fails=$((fails + 1))
        failed="$failed$failed_sep$writer_cmd"
        failed_sep=$nl
        continue
    fi

    # read the trace once
    j=$i
    k=$(((1 << $i) - 1))
    s=$(getsubset $i $k)
    tests=$((tests + 1))
    n=$((n + 1))
    reader_cmd="./test-reader -nt $j $t_nold.otf2 0 $s"
    test ${V-0} -eq 1 && echo "$i.$n - Reading with $j rank(s) |{$s}| out of $i rank(s) (w/o subgroups): $reader_cmd"
    if $VALGRIND $reader_cmd
    then
        successes=$((successes + 1))
    else
        fails=$((fails + 1))
        failed="$failed$writer_cmd$nl$reader_cmd"
        writer_cmd=
    fi

    cleanup
done

echo "Success: $successes / $tests"
test $fails -eq 0 && exit 0
echo "Fails: $fails / $tests"
test ${V-0} -eq 1 && echo "$failed"
exit 1
